@@ -46,7 +46,6 @@
 #include "gena.h"
 #include "genut.h"
 #include "statcodes.h"
-#include "upnp_timeout.h"
 #include "upnpapi.h"
 #include "uri.h"
 
@@ -58,7 +57,7 @@
 #define XMLPARSERTP PicoXMLParser
 #endif
 
-#define UPNP_NOPE (Upnp_LogLevel)(UPNP_ALL+1)
+#define UPNP_NOPE static_cast<Upnp_LogLevel>(UPNP_ALL+1)
 
 extern TimerThread *gTimerThread;
 
@@ -93,21 +92,19 @@ static void clientCancelRenew(ClientSubscription *sub)
     }
 }
 
-struct upnp_timeout_data_subscribe : public upnp_timeout_data {
-    struct Upnp_Event_Subscribe sub;
-};
-
 class AutoRenewSubscriptionJobWorker : public JobWorker {
 public:
-    explicit AutoRenewSubscriptionJobWorker(upnp_timeout *ev)
-        : m_event(ev) {}
-    ~AutoRenewSubscriptionJobWorker() override {
-        deleteZ(m_event);
+    explicit AutoRenewSubscriptionJobWorker(
+        int h, std::string s, int e, const std::string& url, int t) {
+        handle = h;
+        sub.Sid = std::move(s);
+        sub.ErrCode = e;
+        upnp_strlcpy(sub.PublisherUrl, url, NAME_SIZE);
+        sub.TimeOut = t;
     }
-    AutoRenewSubscriptionJobWorker(const AutoRenewSubscriptionJobWorker&) = delete;
-    AutoRenewSubscriptionJobWorker& operator=(const AutoRenewSubscriptionJobWorker&) = delete;
     void work() override;
-    upnp_timeout *m_event;
+    int handle;
+    struct Upnp_Event_Subscribe sub;
 };
 
 /*!
@@ -116,26 +113,24 @@ public:
  */
 void AutoRenewSubscriptionJobWorker::work()
 {
-    auto event = m_event;
-    auto sub_struct = &(dynamic_cast<upnp_timeout_data_subscribe*>(event->Event)->sub);
     int send_callback = 0;
     Upnp_EventType eventType = UPNP_EVENT_AUTORENEWAL_FAILED;
 
 #if AUTO_RENEW_TIME == 0
     // We are compile-time configured for no auto-renewal.
     UpnpPrintf(UPNP_INFO, GENA, __FILE__, __LINE__, "GENA SUB EXPIRED\n");
-    sub_struct->ErrCode = UPNP_E_SUCCESS;
+    sub.ErrCode = UPNP_E_SUCCESS;
     send_callback = 1;
     eventType = UPNP_EVENT_SUBSCRIPTION_EXPIRED;
 #else
     UpnpPrintf(UPNP_DEBUG, GENA, __FILE__, __LINE__, "GENA AUTO RENEW\n");
-    int timeout = sub_struct->TimeOut;
-    int errCode = genaRenewSubscription(event->handle, sub_struct->Sid, &timeout);
-    sub_struct->ErrCode = errCode;
-    sub_struct->TimeOut = timeout;
+    int timeout = sub.TimeOut;
+    int errCode = genaRenewSubscription(handle, sub.Sid, &timeout);
+    sub.ErrCode = errCode;
+    sub.TimeOut = timeout;
     if (errCode != UPNP_E_SUCCESS &&
-        errCode != GENA_E_BAD_SID &&
-        errCode != GENA_E_BAD_HANDLE) {
+        errCode != UPNP_E_INVALID_SID &&
+        errCode != UPNP_E_INVALID_HANDLE) {
         send_callback = 1;
     }
 #endif
@@ -143,7 +138,7 @@ void AutoRenewSubscriptionJobWorker::work()
     if (send_callback) {
         HandleReadLock();
         struct Handle_Info *handle_info;
-        if (GetHandleInfo(event->handle, &handle_info) != HND_CLIENT) {
+        if (GetHandleInfo(handle, &handle_info) != HND_CLIENT) {
             HandleUnlock();
             return;
         }
@@ -151,7 +146,7 @@ void AutoRenewSubscriptionJobWorker::work()
         /* make callback */
         Upnp_FunPtr callback_fun = handle_info->Callback;
         HandleUnlock();
-        callback_fun(eventType, sub_struct, handle_info->Cookie);
+        callback_fun(eventType, &sub, handle_info->Cookie);
     }
 }
 
@@ -171,33 +166,20 @@ static int ScheduleGenaAutoRenew(
     ClientSubscription *sub)
 {
     if (TimeOut == UPNP_INFINITE) {
-        return GENA_SUCCESS;
+        return UPNP_E_SUCCESS;
     }
 
-    auto RenewEventStruct = new upnp_timeout_data_subscribe;
-    auto RenewEvent =  new upnp_timeout;
-
-    /* schedule expire event */
-    RenewEventStruct->sub.Sid = sub->SID;
-    RenewEventStruct->sub.ErrCode = UPNP_E_SUCCESS;
-    upnp_strlcpy(RenewEventStruct->sub.PublisherUrl, sub->eventURL, NAME_SIZE);
-    RenewEventStruct->sub.TimeOut = TimeOut;
-
-    RenewEvent->handle = client_handle;
-    RenewEvent->Event = RenewEventStruct;
-
     /* Schedule the job */
-    auto worker = std::make_unique<AutoRenewSubscriptionJobWorker>(RenewEvent);
+    auto worker = std::make_unique<AutoRenewSubscriptionJobWorker>(
+        client_handle, sub->SID, UPNP_E_SUCCESS, sub->eventURL, TimeOut);
     int return_code = gTimerThread->schedule(
         TimerThread::SHORT_TERM, TimerThread::REL_SEC, TimeOut - AUTO_RENEW_TIME,
-        &(RenewEvent->eventId), std::move(worker));
+        &sub->renewEventId, std::move(worker));
 
     if (return_code != UPNP_E_SUCCESS) {
         return return_code;
     }
-
-    sub->renewEventId = RenewEvent->eventId;
-    return GENA_SUCCESS;
+    return UPNP_E_SUCCESS;
 }
 
 
@@ -263,7 +245,7 @@ static int gena_unsubscribe(
 }
 
 // localaddr is already in inet_ntop-provided dot or ipv6 format
-static std::string myCallbackUrl(NetIF::IPAddr& netaddr)
+static std::string myCallbackUrl(const NetIF::IPAddr& netaddr)
 {
     std::ostringstream oss;
     oss << "http://";
@@ -279,7 +261,7 @@ static std::string myCallbackUrl(NetIF::IPAddr& netaddr)
 }
 
 struct CurlGuard {
-    CurlGuard() = default;
+    CurlGuard(CURL* t, curl_slist* l) : htalk(t), hlist(l) {}
     ~CurlGuard() {
         if (htalk) {
             curl_easy_cleanup(htalk);
@@ -290,8 +272,8 @@ struct CurlGuard {
     }
     CurlGuard(const CurlGuard&) = delete;
     CurlGuard& operator=(const CurlGuard&) = delete;
-    CURL *htalk{nullptr};
-    struct curl_slist *hlist{nullptr};
+    CURL *htalk;
+    struct curl_slist *hlist;
 };
 
 /*!
@@ -350,11 +332,10 @@ static int gena_subscribe(
         return UPNP_E_SOCKET_CONNECT;
     }
 
-    CurlGuard hdls;
     std::map<std::string, std::string> http_headers;
     char curlerrormessage[CURL_ERROR_SIZE];
 
-    hdls.htalk = curl_easy_init();
+    auto hdls = CurlGuard(curl_easy_init(), nullptr);
     curl_easy_setopt(hdls.htalk, CURLOPT_ERRORBUFFER, curlerrormessage);
     curl_easy_setopt(hdls.htalk, CURLOPT_WRITEFUNCTION, write_callback_null_curl);
     curl_easy_setopt(hdls.htalk, CURLOPT_CUSTOMREQUEST, "SUBSCRIBE");
@@ -433,7 +414,7 @@ int genaUnregisterClient(UpnpClient_Handle client_handle)
 
         if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
             HandleUnlock();
-            return GENA_E_BAD_HANDLE;
+            return UPNP_E_INVALID_HANDLE;
         }
         if (handle_info->ClientSubList.empty()) {
             return_code = UPNP_E_SUCCESS;
@@ -464,13 +445,13 @@ int genaUnSubscribe(UpnpClient_Handle client_handle, const std::string& in_sid)
     struct Handle_Info *handle_info;
     if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
         HandleUnlock();
-        return GENA_E_BAD_HANDLE;
+        return UPNP_E_INVALID_HANDLE;
     }
     auto sub = std::find_if(handle_info->ClientSubList.begin(), handle_info->ClientSubList.end(),
                             [in_sid](const ClientSubscription& e){return e.SID == in_sid;});
     if (handle_info->ClientSubList.end() == sub) {
         HandleUnlock();
-        return GENA_E_BAD_SID;
+        return UPNP_E_INVALID_SID;
     }
     auto timeoutms = handle_info->SubsOpsTimeoutMS;
     auto sub_copy = *sub;
@@ -481,13 +462,13 @@ int genaUnSubscribe(UpnpClient_Handle client_handle, const std::string& in_sid)
     HandleLock();
     if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
         HandleUnlock();
-        return GENA_E_BAD_HANDLE;
+        return UPNP_E_INVALID_HANDLE;
     }
     handle_info->ClientSubList.remove_if(
         [in_sid] (const ClientSubscription& e) {return e.SID == in_sid;});
     HandleUnlock();
 
-    return GENA_SUCCESS;
+    return UPNP_E_SUCCESS;
 }
 
 
@@ -497,8 +478,7 @@ int genaSubscribe(
     int *TimeOut,
     std::string *out_sid)
 {
-    int return_code = GENA_SUCCESS;
-    ClientSubscription newSubscription;
+    int return_code = UPNP_E_SUCCESS;
     std::string SID;
     std::string EventURL;
     struct Handle_Info *handle_info;
@@ -509,7 +489,7 @@ int genaSubscribe(
     HandleReadLock();
     /* validate handle */
     if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
-        return_code = GENA_E_BAD_HANDLE;
+        return_code = UPNP_E_INVALID_HANDLE;
         SubscribeLock();
         goto error_handler;
     }
@@ -527,17 +507,14 @@ int genaSubscribe(
     }
 
     if(GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
-        return_code = GENA_E_BAD_HANDLE;
+        return_code = UPNP_E_INVALID_HANDLE;
         goto error_handler;
     }
 
     /* create event url */
     EventURL = PublisherURL;
-    newSubscription.renewEventId = -1;
-    newSubscription.SID = SID;
     out_sid->assign(SID);
-    newSubscription.eventURL = EventURL;
-    handle_info->ClientSubList.push_front(newSubscription);
+    handle_info->ClientSubList.emplace_front(-1, std::move(SID), std::move(EventURL));
 
     /* schedule expiration event */
     return_code = ScheduleGenaAutoRenew(client_handle, *TimeOut,
@@ -562,14 +539,14 @@ int genaRenewSubscription(
     struct Handle_Info *handle_info;
     if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
         HandleUnlock();
-        return GENA_E_BAD_HANDLE;
+        return UPNP_E_INVALID_HANDLE;
     }
 
     auto sub = std::find_if(handle_info->ClientSubList.begin(), handle_info->ClientSubList.end(),
                             [in_sid](const ClientSubscription& e){return e.SID == in_sid;});
     if (handle_info->ClientSubList.end() == sub) {
         HandleUnlock();
-        return GENA_E_BAD_SID;
+        return UPNP_E_INVALID_SID;
     }
     auto timeoutms = handle_info->SubsOpsTimeoutMS;
 
@@ -588,7 +565,7 @@ int genaRenewSubscription(
 
     if (GetHandleInfo(client_handle, &handle_info) != HND_CLIENT) {
         HandleUnlock();
-        return GENA_E_BAD_HANDLE;
+        return UPNP_E_INVALID_HANDLE;
     }
 
     if (return_code != UPNP_E_SUCCESS) {
@@ -606,7 +583,7 @@ int genaRenewSubscription(
     if (handle_info->ClientSubList.end() == sub) {
         clientCancelRenew(&sub_copy);
         HandleUnlock();
-        return GENA_E_BAD_SID;
+        return UPNP_E_INVALID_SID;
     }
 
     /* Remember SID */
@@ -614,7 +591,7 @@ int genaRenewSubscription(
 
     /* start renew subscription timer */
     return_code = ScheduleGenaAutoRenew(client_handle, *TimeOut, &*sub);
-    if (return_code != GENA_SUCCESS) {
+    if (return_code != UPNP_E_SUCCESS) {
         handle_info->ClientSubList.remove_if(
             [sub] (const ClientSubscription& e) {return e.SID == sub->SID;});
     }
