@@ -54,7 +54,7 @@ static constexpr auto XML_PROPERTYSET_HEADER =
 /*!
  * \brief Unregisters a device.
  *
- * \return UPNP_E_SUCCESS on success, GENA_E_BAD_HANDLE on failure.
+ * \return UPNP_E_SUCCESS on success, UPNP_E_INVALID_HANDLE on failure.
  */
 int genaUnregisterDevice(UpnpDevice_Handle device_handle)
 {
@@ -65,9 +65,9 @@ int genaUnregisterDevice(UpnpDevice_Handle device_handle)
     if (GetHandleInfo(device_handle, &handle_info) != HND_DEVICE) {
         UpnpPrintf(UPNP_CRITICAL, GENA, __FILE__, __LINE__,
             "genaUnregisterDevice: BAD Handle: %d\n", device_handle);
-        ret = GENA_E_BAD_HANDLE;
+        ret = UPNP_E_INVALID_HANDLE;
     } else {
-        freeServiceTable(&handle_info->ServiceTable);
+        clearServiceTable(handle_info->serviceTable);
         ret = UPNP_E_SUCCESS;
     }
     HandleUnlock();
@@ -78,7 +78,7 @@ int genaUnregisterDevice(UpnpDevice_Handle device_handle)
 /*!
  * \brief Generates XML property set for notifications.
  *
- * \return UPNP_E_SUCCESS if successful else returns GENA_E_BAD_HANDLE.
+ * \return UPNP_E_SUCCESS if successful else returns UPNP_E_INVALID_HANDLE.
  *
  * \note The XML_VERSION comment is NOT sent due to interoperability issues
  *     with other UPnP vendors.
@@ -113,7 +113,7 @@ static int GeneratePropertySet(
  *
  * NOTIFY http request is sent and the reply is processed.
  *
- * \return GENA_SUCCESS if the event was delivered, otherwise returns the
+ * \return UPNP_E_SUCCESS if the event was delivered, otherwise returns the
  *     appropriate error code.
  *
  * The only code which has specific processing is
@@ -127,10 +127,9 @@ static int GeneratePropertySet(
  */
 static int genaNotify(const std::string& propertySet, const subscription *sub)
 {
-    std::string mid_msg;
     int return_code = -1;
-
     long http_code = 0;
+
     /* send a notify to each url until one goes thru */
     for (const auto& url : sub->DeliveryURLs) {
         CURL *easy = curl_easy_init();
@@ -179,13 +178,13 @@ static int genaNotify(const std::string& propertySet, const subscription *sub)
 
     if (return_code == UPNP_E_SUCCESS) {
         if (http_code == HTTP_OK) {
-            return_code = GENA_SUCCESS;
+            return_code = UPNP_E_SUCCESS;
         } else {
             if (http_code == HTTP_PRECONDITION_FAILED)
                 /*Invalid SID gets removed */
                 return_code = GENA_E_NOTIFY_UNACCEPTED_REMOVE_SUB;
             else
-                return_code = GENA_E_NOTIFY_UNACCEPTED;
+                return_code = UPNP_E_NOTIFY_UNACCEPTED;
         }
     }
     return return_code;
@@ -198,6 +197,10 @@ static int genaNotify(const std::string& propertySet, const subscription *sub)
    though (e.g. with shared_ptr for the propertySet string. We don't
    typically have thousands of subscribed CPs... */
 struct Notification {
+    Notification(std::string sid, std::string udn, std::string pset, Upnp_SID uSID,
+                 time_t ct, UpnpDevice_Handle dh)
+        : device_handle(dh), UDN(std::move(udn)), servId(std::move(sid)), sid(std::move(uSID)),
+          propertySet(std::move(pset)), ctime(ct) {}
     UpnpDevice_Handle device_handle; //
     std::string UDN;                 // Device
     std::string servId;  // Service
@@ -208,15 +211,16 @@ struct Notification {
 
 class GenaNotifyJobWorker : public JobWorker {
 public:
-    explicit GenaNotifyJobWorker(Notification *in)
-        : m_input(in) {}
-    ~GenaNotifyJobWorker() override {
-        delete m_input;
-    }
-    GenaNotifyJobWorker(const GenaNotifyJobWorker&) = delete;
-    GenaNotifyJobWorker& operator=(const GenaNotifyJobWorker&) = delete;
+    explicit GenaNotifyJobWorker(std::shared_ptr<Notification> in)
+        : m_input(std::move(in)) {}
     void work() override;
-    Notification *m_input;
+    // This is actually shared with the outgoing list head.  Note: 2024-02: as far as I understand,
+    // the only reason to keep a shared pointer to the active notification at the head of the
+    // subscription outgoing queue is as an indicator that we don't need to kickstart the
+    // queue. Else when adding an event and seeing an empty queue, we would not know if the event
+    // should be queued or directly sent to the Thread Pool. This could probably be replaced by a
+    // flag, which would allow to replace the shared_ptr with unique_ptr.
+    std::shared_ptr<Notification> m_input;
 };
 
 /*!
@@ -246,8 +250,7 @@ void GenaNotifyJobWorker::work()
         return;
     }
 
-    if (!(service = FindServiceId(&handle_info->ServiceTable,
-                                  m_input->servId, m_input->UDN)) ||
+    if (!(service = FindServiceId(handle_info->serviceTable, m_input->servId, m_input->UDN)) ||
         !service->active ||
         !(sub = GetSubscriptionSID(m_input->sid, service)) ||
         copy_subscription(sub, &sub_copy) != UPNP_E_SUCCESS) {
@@ -265,8 +268,7 @@ void GenaNotifyJobWorker::work()
         return;
     }
     /* validate context */
-    if (!(service = FindServiceId(&handle_info->ServiceTable,
-                                  m_input->servId, m_input->UDN)) ||
+    if (!(service = FindServiceId(handle_info->serviceTable, m_input->servId, m_input->UDN)) ||
         !service->active ||
         !(sub = GetSubscriptionSID(m_input->sid, service))) {
         HandleUnlock();
@@ -277,8 +279,7 @@ void GenaNotifyJobWorker::work()
         /* wrap to 1 for overflow */
         sub->ToSendEventKey = 1;
 
-    /* Remove head of event queue. Do not delete it, the ThreadJob free_func
-       will do it */
+    /* Remove head of event queue. */
     if (!sub->outgoing.empty()) {
         sub->outgoing.pop_front();
     }
@@ -308,11 +309,9 @@ int genaInitNotifyXML(
     const std::string& propertySet,
     const Upnp_SID& sid)
 {
-    int ret = GENA_SUCCESS;
+    int ret = UPNP_E_SUCCESS;
     int line = 0;
-
-    Notification *thread_struct = nullptr;
-
+    std::shared_ptr<Notification> thread_struct;
     subscription *sub = nullptr;
     service_info *service = nullptr;
     struct Handle_Info *handle_info;
@@ -324,39 +323,28 @@ int genaInitNotifyXML(
 
     if (GetHandleInfo(device_handle, &handle_info) != HND_DEVICE) {
         line = __LINE__;
-        ret = GENA_E_BAD_HANDLE;
+        ret = UPNP_E_INVALID_HANDLE;
         goto ExitFunction;
     }
 
-    service = FindServiceId(&handle_info->ServiceTable, servId, UDN);
+    service = FindServiceId(handle_info->serviceTable, servId, UDN);
     if (service == nullptr) {
         line = __LINE__;
-        ret = GENA_E_BAD_SERVICE;
+        ret = UPNP_E_INVALID_SERVICE;
         goto ExitFunction;
     }
 
     sub = GetSubscriptionSID(sid, service);
     if (sub == nullptr || sub->active) {
         line = __LINE__;
-        ret = GENA_E_BAD_SID;
+        ret = UPNP_E_INVALID_SID;
         goto ExitFunction;
     }
     sub->active = 1;
 
     /* schedule thread for initial notification */
-    thread_struct = new Notification;
-    if (thread_struct == nullptr) {
-        line = __LINE__;
-        ret = UPNP_E_OUTOF_MEMORY;
-        goto ExitFunction;
-    }
-    thread_struct->servId = servId;
-    thread_struct->UDN = UDN;
-    thread_struct->propertySet = propertySet;
-    thread_struct->sid = sid;
-    thread_struct->ctime = time(nullptr);
-    thread_struct->device_handle = device_handle;
-
+    thread_struct = std::make_shared<Notification>(
+        servId, UDN, propertySet, sid, time(nullptr), device_handle);
     {
         auto worker = std::make_unique<GenaNotifyJobWorker>(thread_struct);
         ret = gSendThreadPool.addJob(std::move(worker));
@@ -367,16 +355,12 @@ int genaInitNotifyXML(
     } else {
         line = __LINE__;
         sub->outgoing.push_back(thread_struct);
-        ret = GENA_SUCCESS;
+        ret = UPNP_E_SUCCESS;
     }
 
 ExitFunction:
-    if (ret != GENA_SUCCESS) {
-        delete thread_struct;
-    }
     HandleUnlock();
-    UpnpPrintf(UPNP_ALL, GENA, __FILE__, line,
-               "genaInitNotifyCommon: ret %d\n", ret);
+    UpnpPrintf(UPNP_ALL, GENA, __FILE__, line, "genaInitNotifyCommon: ret %d\n", ret);
     return ret;
 }
 
@@ -389,16 +373,15 @@ int genaInitNotifyVars(
     int var_count,
     const Upnp_SID& sid)
 {
-    int ret = GENA_SUCCESS;
+    int ret = UPNP_E_SUCCESS;
     int line = 0;
     std::string propertySet;
 
-    UpnpPrintf(UPNP_DEBUG, GENA, __FILE__, __LINE__,
-               "genaInitNotifyVars varcnt %d\n", var_count);
+    UpnpPrintf(UPNP_DEBUG, GENA, __FILE__, __LINE__, "genaInitNotifyVars varcnt %d\n", var_count);
 
     if (var_count <= 0) {
         line = __LINE__;
-        ret = GENA_SUCCESS;
+        ret = UPNP_E_SUCCESS;
         goto ExitFunction;
     }
 
@@ -415,21 +398,6 @@ ExitFunction:
     return ret;
 }
 
-void freeSubscriptionQueuedEvents(subscription *sub)
-{
-    /* The first event is discarded without deleting the Notification:
-       there is a mirror ThreadPool entry for this one, and the
-       completion event will take care of it. Other entries must be
-       fully cleaned-up here */
-    auto it = sub->outgoing.begin();
-    if (it != sub->outgoing.end())
-        it++;
-    while (it != sub->outgoing.end()) {
-        delete *it;
-        it = sub->outgoing.erase(it);
-    }
-}
-
 /*
  * This gets called before queuing a new event, with the handLock held.
  * - The list size can never go over MAX_SUBSCRIPTION_QUEUED_EVENTS so we
@@ -438,7 +406,7 @@ void freeSubscriptionQueuedEvents(subscription *sub)
  * non-active: any but the head of queue, which is already copied to
  * the thread pool
  */
-static void maybeDiscardEvents(std::list<Notification*>& outgoing)
+static void maybeDiscardEvents(std::list<std::shared_ptr<Notification>>& outgoing)
 {
     time_t now = time(nullptr);
 
@@ -447,10 +415,8 @@ static void maybeDiscardEvents(std::list<Notification*>& outgoing)
     if (it != outgoing.end())
         it++;
     while (it != outgoing.end()) {
-        Notification *ntsp = *it;
         if (outgoing.size() > unsigned(g_UpnpSdkEQMaxLen) ||
-            now - ntsp->ctime > g_UpnpSdkEQMaxAge) {
-            delete ntsp;
+            now - (*it)->ctime > g_UpnpSdkEQMaxAge) {
             it = outgoing.erase(it);
         } else {
             /* If the list is smaller than the max and the oldest
@@ -461,15 +427,12 @@ static void maybeDiscardEvents(std::list<Notification*>& outgoing)
 }
 
 int genaNotifyAllXML(
-    UpnpDevice_Handle device_handle,
-    char *UDN, char *servId, const std::string& propertySet)
+    UpnpDevice_Handle device_handle, char *UDN, char *servId, const std::string& propertySet)
 {
-    int ret = GENA_SUCCESS;
+    int ret = UPNP_E_SUCCESS;
     int line = 0;
     std::list<subscription>::iterator finger;
-
-    Notification *thread_struct = nullptr;
-
+    std::shared_ptr<Notification> thread_struct;
     service_info *service = nullptr;
     struct Handle_Info *handle_info;
 
@@ -480,37 +443,25 @@ int genaNotifyAllXML(
 
     if (GetHandleInfo(device_handle, &handle_info) != HND_DEVICE) {
         line = __LINE__;
-        ret = GENA_E_BAD_HANDLE;
+        ret = UPNP_E_INVALID_HANDLE;
         goto ExitFunction;
     }
 
-    service = FindServiceId(&handle_info->ServiceTable, servId, UDN);
+    service = FindServiceId(handle_info->serviceTable, servId, UDN);
     if (service == nullptr) {
         line = __LINE__;
-        ret = GENA_E_BAD_SERVICE;
+        ret = UPNP_E_INVALID_SERVICE;
         goto ExitFunction;
     }
 
     finger = GetFirstSubscription(service);
     while (finger != service->subscriptionList.end()) {
-        thread_struct = new Notification;
-        if (thread_struct == nullptr) {
-            line = __LINE__;
-            ret = UPNP_E_OUTOF_MEMORY;
-            break;
-        }
-        thread_struct->UDN = UDN;
-        thread_struct->servId = servId;
-        thread_struct->propertySet = propertySet;
-        thread_struct->ctime = time(nullptr);
-        thread_struct->device_handle = device_handle;
-        thread_struct->sid = finger->sid;
-
+        thread_struct = std::make_shared<Notification>(
+            servId, UDN, propertySet, finger->sid, time(nullptr), device_handle);
         maybeDiscardEvents(finger->outgoing);
         finger->outgoing.push_back(thread_struct);
 
-        /* If there is only one element on the list (which we just
-           added), need to kickstart the threadpool */
+        /* If there is only one element on the list (just added), kickstart the threadpool */
         if (finger->outgoing.size() == 1) {
             auto worker = std::make_unique<GenaNotifyJobWorker>(thread_struct);
             ret = gSendThreadPool.addJob(std::move(worker));
@@ -528,10 +479,7 @@ int genaNotifyAllXML(
 
 ExitFunction:
     HandleUnlock();
-
-    UpnpPrintf(UPNP_ALL, GENA, __FILE__, line,
-               "genaNotifyAllCommon: ret = %d\n", ret);
-
+    UpnpPrintf(UPNP_ALL, GENA, __FILE__, line, "genaNotifyAllCommon: ret = %d\n", ret);
     return ret;
 }
 
@@ -543,7 +491,7 @@ int genaNotifyAll(
     char **VarValues,
     int var_count)
 {
-    int ret = GENA_SUCCESS;
+    int ret = UPNP_E_SUCCESS;
     int line = 0;
 
     UpnpPrintf(UPNP_ALL, GENA, __FILE__, __LINE__, "genaNotifyAll\n");
@@ -601,7 +549,7 @@ static int respond_ok(MHDTransaction *mhdt, int time_out, subscription *sub,
 // one of our supported addresses, not necessarily the one this
 // particular client connected to.
 static bool callStrangerCheck(
-    const std::string& surl, uri_type& temp, const NetIF::Interface *clnetif, NetIF::IPAddr& claddr)
+    const std::string& surl, uri_type& temp, const NetIF::Interface *clnetif, const NetIF::IPAddr& claddr)
 {
     NetIF::IPAddr subsaddr(reinterpret_cast<struct sockaddr*>(&temp.hostport.IPaddress));
     if (!subsaddr.ok()) {
@@ -649,7 +597,7 @@ static int create_url_list(
     out->clear();
 
     // Get information for the client address
-    NetIF::IPAddr claddr(reinterpret_cast<struct sockaddr*>(mhdt->client_address));
+    NetIF::IPAddr claddr(reinterpret_cast<struct sockaddr*>(&mhdt->client_address));
     if (!claddr.ok()) {
         UpnpPrintf(UPNP_INFO, GENA, __FILE__, __LINE__,
                    "create_url_list: can't determine client addr\n");
@@ -666,7 +614,7 @@ static int create_url_list(
         return UPNP_E_INVALID_INTERFACE;
     }
 
-    std::string::size_type openpos = 0;
+    std::string::size_type openpos;
     std::string::size_type closepos = 0;
     for (;;) {
         if ((openpos = ulist.find('<', closepos)) == std::string::npos) {
@@ -717,7 +665,6 @@ static int create_url_list(
 
 void gena_process_subscription_request(MHDTransaction *mhdt)
 {
-    struct Upnp_Subscription_Request request_struct = {};
     int return_code = 1;
     int time_out = 1801;
     service_info *service;
@@ -725,7 +672,6 @@ void gena_process_subscription_request(MHDTransaction *mhdt)
     void *cookie;
     Upnp_FunPtr callback_fun;
     UpnpDevice_Handle device_handle;
-    std::list<subscription>::iterator sub;
 
     UpnpPrintf(UPNP_INFO, GENA, __FILE__, __LINE__,
                "Subscription Request Received:\n");
@@ -796,7 +742,7 @@ void gena_process_subscription_request(MHDTransaction *mhdt)
     }
 
     /* generate new subscription */
-    sub = service->subscriptionList.emplace(service->subscriptionList.end());
+    auto sub = service->subscriptionList.emplace(service->subscriptionList.end());
     if (sub == service->subscriptionList.end()) {
         http_SendStatusResponse(mhdt, HTTP_INTERNAL_SERVER_ERROR);
         HandleUnlock();
@@ -836,9 +782,7 @@ void gena_process_subscription_request(MHDTransaction *mhdt)
                "Subscription Request granted\n");
 
     /* finally generate callback for init table dump */
-    request_struct.ServiceId = service->serviceId.c_str();
-    request_struct.UDN = service->UDN.c_str();
-    request_struct.Sid = sub->sid;
+    Upnp_Subscription_Request request_struct{service->serviceId.c_str(), service->UDN.c_str(), sub->sid};
 
     /* copy callback */
     callback_fun = handle_info->Callback;
@@ -859,7 +803,6 @@ void gena_process_subscription_renewal_request(MHDTransaction *mhdt)
     service_info *service;
     struct Handle_Info *handle_info;
     UpnpDevice_Handle device_handle;
-    std::string event_url_path;
 
     /* if a CALLBACK or NT header is present, then it is an error */
     if (mhdt->headers.find("callback") != mhdt->headers.end() ||
