@@ -74,7 +74,7 @@ struct MiniServerSockArray {
     MiniServerSockArray& operator=(const MiniServerSockArray&) = delete;
 
 private:
-    void maybeClose(SOCKET s) {
+    static void maybeClose(SOCKET s) {
         if (s != INVALID_SOCKET) {
             UpnpCloseSocket(s);
         }
@@ -99,7 +99,7 @@ void SetSoapCallback(
     /*! [in] SOAP Callback to be invoked . */
     MiniServerCallback callback);
 #else /* INCLUDE_DEVICE_APIS */
-    static UPNP_INLINE void SetSoapCallback(MiniServerCallback callback) {}
+static inline void SetSoapCallback(MiniServerCallback callback) {}
 #endif /* INCLUDE_DEVICE_APIS */
 
 /*!
