@@ -65,7 +65,7 @@ extern size_t g_maxContentLength;
 extern int g_UpnpSdkEQMaxLen;
 extern int g_UpnpSdkEQMaxAge;
 
-typedef enum {HND_INVALID=-1,HND_CLIENT,HND_DEVICE} Upnp_Handle_Type;
+typedef enum {HND_INVALID=-1, HND_CLIENT, HND_DEVICE} Upnp_Handle_Type;
 
 struct SsdpSearchArg;
 
@@ -75,13 +75,11 @@ struct Handle_Info
     Handle_Info() = default;
     ~Handle_Info() = default;
     /*! . */
-    Upnp_Handle_Type HType{Upnp_Handle_Type(0)};
+    Upnp_Handle_Type HType{HND_INVALID};
     /*! Callback function pointer. */
     Upnp_FunPtr  Callback{nullptr};
     /*! . */
-    char *Cookie{nullptr};
-    /*! 0 = not installed; otherwise installed. */
-    int   aliasInstalled;
+    char* Cookie{nullptr};
 
     /* Device Only */
 #ifdef INCLUDE_DEVICE_APIS
@@ -102,8 +100,8 @@ struct Handle_Info
     int RegistrationState{0};
     /*! Parsed Device Description document. */
     UPnPDeviceDesc devdesc;
-    /*! Table holding subscriptions and URL information. */
-    service_table ServiceTable;
+    /*! Service information and subscriptions lists */
+    std::list<service_info> serviceTable;
     /*! . */
     int MaxSubscriptions{0};
     /*! . */
@@ -115,7 +113,7 @@ struct Handle_Info
     /*! Client subscription list. */
     std::list<ClientSubscription> ClientSubList;
     /*! Active SSDP searches. */
-    std::list<std::unique_ptr<SsdpSearchArg>> SsdpSearchList;
+    std::list<SsdpSearchArg> SsdpSearchList;
     int SubsOpsTimeoutMS{HTTP_DEFAULT_TIMEOUT * 1000};
 #endif
 
