@@ -8,11 +8,7 @@
 #include <winsock2.h>
 #include <ws2tcpip.h>
 
-#if(_WIN32_WINNT < 0x0600)
-typedef short sa_family_t;
-#else
 typedef ADDRESS_FAMILY sa_family_t;
-#endif
 
 #define UpnpCloseSocket(s)  do {closesocket(s); s = INVALID_SOCKET;} while(0)
 #define UPNP_SOCK_GET_LAST_ERROR() WSAGetLastError()
@@ -33,11 +29,11 @@ typedef int SOCKET;
 #  include <sys/sockio.h>
 #endif
 
-#include <sys/types.h>
-#include <sys/socket.h>
+#include <arpa/inet.h>
 #include <ifaddrs.h>
 #include <netinet/in.h>
-#include <arpa/inet.h>
+#include <sys/socket.h>
+#include <sys/types.h>
 
 
 #endif /* ! _WIN32 */
