@@ -107,6 +107,12 @@ public:
     struct sockaddr *saddr() {
         return reinterpret_cast<struct sockaddr*>(&address);
     }
+    struct sockaddr_in *saddrin() {
+        return reinterpret_cast<struct sockaddr_in*>(&address);
+    }
+    struct sockaddr_in6 *saddrin6() {
+        return reinterpret_cast<struct sockaddr_in6*>(&address);
+    }
 };
 
 IPAddr::IPAddr()
@@ -141,15 +147,13 @@ IPAddr& IPAddr::operator=(IPAddr&& o) noexcept {
 IPAddr::IPAddr(const char *caddr)
     : IPAddr()
 {
-    if (nullptr != std::strchr(caddr, ':')) {
-        if (inet_pton(AF_INET6, caddr,
-                      &reinterpret_cast<struct sockaddr_in6*>(m->saddr())->sin6_addr) == 1) {
+    if (std::strchr(caddr, ':')) {
+        if (inet_pton(AF_INET6, caddr, &m->saddrin6()->sin6_addr) == 1) {
             m->saddr()->sa_family = AF_INET6;
             m->ok = true;
         }
     } else {
-        if (inet_pton(AF_INET, caddr,
-                      &reinterpret_cast<struct sockaddr_in*>(m->saddr())->sin_addr) == 1) {
+        if (inet_pton(AF_INET, caddr, &m->saddrin()->sin_addr) == 1) {
             m->saddr()->sa_family = AF_INET;
             m->ok = true;
         }
@@ -169,10 +173,11 @@ IPAddr::IPAddr(const struct sockaddr *sa, bool unmapv4)
     case AF_INET6:
     {
         if (unmapv4) {
-            const uint8_t *bytes =
-                reinterpret_cast<const struct sockaddr_in6 *>(sa)->sin6_addr.s6_addr;
+            sockaddr_in6 sa6;
+            std::memcpy(&sa6, sa, sizeof(sockaddr_in6));
+            const uint8_t* bytes = sa6.sin6_addr.s6_addr;
             if (!memcmp(bytes, ipv4mappedprefix, 12)) {
-                auto a = reinterpret_cast<struct sockaddr_in*>(m->saddr());
+                auto a = m->saddrin();
                 memset(a, 0, sizeof(*a));
                 a->sin_family = AF_INET;
                 memcpy(&a->sin_addr.s_addr, bytes+12, 4);
@@ -250,8 +255,7 @@ IPAddr::Scope IPAddr::scopetype() const
     // necessary to determine what interface they belong too.
     // e.g. fe80::1 could exist on both eth0 and eth1, and needs
     // scopeid 0/1 for complete determination
-    if (IN6_IS_ADDR_LINKLOCAL(
-            &(reinterpret_cast<struct sockaddr_in6*>(m->saddr()))->sin6_addr)) {
+    if (IN6_IS_ADDR_LINKLOCAL(&m->saddrin6()->sin6_addr)) {
         return Scope::LINK;
     }
 
@@ -259,8 +263,7 @@ IPAddr::Scope IPAddr::scopetype() const
     // chosen network and interface ids. Routable within the
     // site. They also need a site/scope ID, always 1 if there is only
     // one site defined.
-    if (IN6_IS_ADDR_SITELOCAL(
-            &(reinterpret_cast<struct sockaddr_in6*>(m->saddr()))->sin6_addr)) {
+    if (IN6_IS_ADDR_SITELOCAL(&m->saddrin6()->sin6_addr)) {
         return Scope::SITE;
     }
 
@@ -274,8 +277,8 @@ bool IPAddr::setScopeIdx(const IPAddr& other)
         scopetype() != Scope::LINK || other.scopetype() != Scope::LINK) {
         return false;
     }
-    auto msa6 = reinterpret_cast<struct sockaddr_in6*>(m->saddr());
-    auto osa6 = reinterpret_cast<struct sockaddr_in6*>(other.m->saddr());
+    auto msa6 = m->saddrin6();
+    auto osa6 = other.m->saddrin6();
     msa6->sin6_scope_id = osa6->sin6_scope_id;
     return true;
 }
@@ -294,12 +297,11 @@ std::string IPAddr::straddr(bool setscope, bool forurl) const
     buf[0] = 0;
     switch(m->saddr()->sa_family) {
     case AF_INET:
-        inet_ntop(m->saddr()->sa_family,
-                  &reinterpret_cast<struct sockaddr_in*>(m->saddr())->sin_addr, buf, 200);
-    break;
+        inet_ntop(m->saddr()->sa_family, &m->saddrin()->sin_addr, buf, 200);
+        break;
     case AF_INET6:
     {
-        auto sa6 = reinterpret_cast<struct sockaddr_in6*>(m->saddr());
+        auto sa6 = m->saddrin6();
         inet_ntop(m->saddr()->sa_family, &sa6->sin6_addr, buf, 200);
         if (!setscope || scopetype() != Scope::LINK) {
             return buf;
@@ -427,16 +429,18 @@ bool Interface::trimto(const std::vector<IPAddr>& keep)
 {
     auto mit = m->netmasks.begin();
     for (auto ait = m->addresses.begin(); ait != m->addresses.end();) {
-        auto it = find_if(keep.begin(), keep.end(),
-                          [ait] (const IPAddr& a) {
-                              return ait->straddr() == a.straddr();
-                          });
-        if (it == keep.end()) {
+        bool found = false;
+        for (const auto& a : keep) {
+            if (ait->straddr() == a.straddr()) {
+                ait++;
+                mit++;
+                found = true;
+                break;
+            }
+        }
+        if (!found) {
             ait = m->addresses.erase(ait);
             mit = m->netmasks.erase(mit);
-        } else {
-            ait++;
-            mit++;
         }
     }
     return !m->addresses.empty();
@@ -463,8 +467,7 @@ const IPAddr *Interface::firstipv6addr(IPAddr::Scope scope) const
     for (const auto& entry: m->addresses) {
         if (entry.family() == IPAddr::Family::IPV6 &&
             (scope != IPAddr::Scope::LINK ||
-             IN6_IS_ADDR_LINKLOCAL(
-                 &(reinterpret_cast<struct sockaddr_in6*>(entry.m->saddr()))->sin6_addr))) {
+             IN6_IS_ADDR_LINKLOCAL(&entry.m->saddrin6()->sin6_addr))) {
             return &entry;
         }
     }
@@ -573,8 +576,9 @@ Interfaces::Internal::Internal()
 #if defined(__linux__)
         case AF_PACKET:
         {
-            auto sll = reinterpret_cast<struct sockaddr_ll*>(ifa->ifa_addr);
-            ifit->m->sethwaddr(reinterpret_cast<const char*>(sll->sll_addr), sll->sll_halen);
+            sockaddr_ll sll;
+            std::memcpy(&sll, ifa->ifa_addr, sizeof(sockaddr_ll));
+            ifit->m->sethwaddr(reinterpret_cast<const char*>(sll.sll_addr), sll.sll_halen);
         }
         break;
 #elif !defined(__CYGWIN__)
@@ -582,7 +586,7 @@ Interfaces::Internal::Internal()
         {
             auto sdl = reinterpret_cast<struct sockaddr_dl*>(ifa->ifa_addr);
             LOGDEB("NetIF::Interfaces: " << ifa->ifa_name << " has hwaddr\n");
-            ifit->m->sethwaddr((const char*)LLADDR(sdl), sdl->sdl_alen);
+            ifit->m->sethwaddr(reinterpret_cast<const char*>(LLADDR(sdl)), sdl->sdl_alen);
         }
         break;
 #endif
@@ -615,7 +619,7 @@ static bool wchartoutf8(const wchar_t* in, std::string& out, size_t wlen)
         fwprintf(stderr, L"wchartoutf8: conversion error1 for [%s]\n", in);
         return false;
     }
-    char *cp = (char *)malloc(bytes+1);
+    auto cp = static_cast<char *>(malloc(bytes+1));
     if (nullptr == cp) {
         LOGERR("wchartoutf8: malloc failed\n");
         return false;
@@ -705,7 +709,7 @@ Interfaces::Internal::Internal()
         // Note: upnpapi.c used IfIndex instead.
         ifit->m->index = adapts_item->Ipv6IfIndex;
         /* The MAC is in the pAdapter->Address char array */
-        ifit->m->sethwaddr((const char *)adapts_item->PhysicalAddress,
+        ifit->m->sethwaddr(reinterpret_cast<const char *>(adapts_item->PhysicalAddress),
                            adapts_item->PhysicalAddressLength);
         LOGDEB("NetIF::Interfaces: " << tmpnm << " has hwaddr\n");
         uni_addr = adapts_item->FirstUnicastAddress;
@@ -721,7 +725,7 @@ Interfaces::Internal::Internal()
                 struct sockaddr_in sa = {};
                 sa.sin_family = AF_INET;
                 sa.sin_addr.s_addr = mask;
-                ifit->m->netmasks.emplace_back((struct sockaddr*)&sa);
+                ifit->m->netmasks.emplace_back(reinterpret_cast<sockaddr*>(&sa));
             }
             break;
             case AF_INET6:
@@ -805,11 +809,9 @@ std::vector<Interface> Interfaces::select(const Filter& filt) const
            " noflags " << noflags << std::dec << "\n");
 
     std::vector<Interface> out;
-    const auto& ifs = theInterfaces()->m->interfaces;
-    std::copy_if(ifs.begin(), ifs.end(), std::back_inserter(out),
-        [=](const NetIF::Interface &entry){
-                     return (entry.m->flags & yesflags) == yesflags &&
-                         (entry.m->flags & noflags) == 0;});
+    for (const auto& entry : theInterfaces()->m->interfaces)
+        if ((entry.m->flags & yesflags) == yesflags && (entry.m->flags & noflags) == 0)
+            out.push_back(entry);
     return out;
 }
 
@@ -827,11 +829,11 @@ static const Interface* interfaceForAddress4(
 {
     struct sockaddr_storage sbuf, mbuf;
     for (const auto& netif : vifs) {
-        auto addresses = netif.getaddresses();
-        for (unsigned int i = 0; i < addresses.first.size(); i++) {
-            if (addresses.first[i].family() == IPAddr::Family::IPV4) {
-                addresses.first[i].copyToStorage(&sbuf);
-                addresses.second[i].copyToStorage(&mbuf);
+        auto [addrs, masks] = netif.getaddresses();
+        for (unsigned int i = 0; i < addrs.size(); i++) {
+            if (addrs[i].family() == IPAddr::Family::IPV4) {
+                addrs[i].copyToStorage(&sbuf);
+                masks[i].copyToStorage(&mbuf);
                 uint32_t addr = reinterpret_cast<struct sockaddr_in*>(&sbuf)->sin_addr.s_addr;
                 uint32_t mask = reinterpret_cast<struct sockaddr_in*>(&mbuf)->sin_addr.s_addr;
                 if (mask == 0) // ??
@@ -843,7 +845,7 @@ static const Interface* interfaceForAddress4(
                     ((vifs.size() == 1) && (mask == 0xffffffff)) ||
                     // Normal subnet check
                     ((peeraddr & mask) == (addr & mask)) ) {
-                    hostaddr = addresses.first[i];
+                    hostaddr = addrs[i];
                     return &netif;
                 }
             }
